@@ -18,11 +18,11 @@ systems. The pages are grouped in numbered sections:
  7 miscellaneous
  8 system administration
 
-Man pages are written *troff*, a text file formatting system.
+Man pages are written in *troff*, a text file formatting system.
 
 See http://www.tldp.org/HOWTO/Man-Page for a start.
 
-Man pages have no subsection only parts.
+Man pages have no subsections only parts.
 Standard parts
 
   NAME ,
@@ -41,12 +41,18 @@ A unix-like system keeps an index of the DESCRIPTIONs, which is accesable
 by the command whatis or apropos.
 
 """
+from __future__ import absolute_import
 
 __docformat__ = 'reStructuredText'
 
+import inspect
 import re
 
-from docutils import nodes, writers, languages
+from docutils import (
+    languages,
+    nodes,
+    writers,
+)
 try:
     import roman
 except ImportError:
@@ -317,7 +323,7 @@ class Translator(nodes.NodeVisitor):
                     self._cnt = 0
                 self._indent = 2
                 if style == 'arabic':
-                    # indentation depends on number of childrens
+                    # indentation depends on number of children
                     # and start value.
                     self._indent = len(str(len(node.children)))
                     self._indent += len(str(self._cnt)) + 1
@@ -427,7 +433,7 @@ class Translator(nodes.NodeVisitor):
         pass
 
     def visit_block_quote(self, node):
-        # BUG/HACK: indent alway uses the _last_ indention,
+        # BUG/HACK: indent always uses the _last_ indentation,
         # thus we need two of them.
         self.indent(BLOCKQOUTE_INDENT)
         self.indent(0)
