@@ -1,18 +1,32 @@
-from pprint import pprint
-from mercurial import minirst
+from __future__ import absolute_import, print_function
+import pprint
+from mercurial import (
+    minirst,
+)
 
-def debugformat(title, text, width, **kwargs):
-    print "%s formatted to fit within %d characters:" % (title, width)
-    print "-" * 70
-    formatted = minirst.format(text, width, **kwargs)
-    if type(formatted) == tuple:
-        print formatted[0]
-        print "-" * 70
-        pprint(formatted[1])
+def debugformat(text, form, **kwargs):
+    if form == 'html':
+        print("html format:")
+        out = minirst.format(text, style=form, **kwargs)
     else:
-        print formatted
-    print "-" * 70
-    print
+        print("%d column format:" % form)
+        out = minirst.format(text, width=form, **kwargs)
+
+    print("-" * 70)
+    if type(out) == tuple:
+        print(out[0][:-1])
+        print("-" * 70)
+        pprint.pprint(out[1])
+    else:
+        print(out[:-1])
+    print("-" * 70)
+    print()
+
+def debugformats(title, text, **kwargs):
+    print("== %s ==" % title)
+    debugformat(text, 60, **kwargs)
+    debugformat(text, 30, **kwargs)
+    debugformat(text, 'html', **kwargs)
 
 paragraphs = """
 This is some text in the first paragraph.
@@ -23,9 +37,7 @@ This is some text in the first paragraph.
  \n  \n   \nThe third and final paragraph.
 """
 
-debugformat('paragraphs', paragraphs, 60)
-debugformat('paragraphs', paragraphs, 30)
-
+debugformats('paragraphs', paragraphs)
 
 definitions = """
 A Term
@@ -40,9 +52,7 @@ Another Term
     Definition.
 """
 
-debugformat('definitions', definitions, 60)
-debugformat('definitions', definitions, 30)
-
+debugformats('definitions', definitions)
 
 literals = r"""
 The fully minimized form is the most
@@ -66,9 +76,7 @@ simply ends with space-double-colon. ::
       with '::' disappears in the final output.
 """
 
-debugformat('literals', literals, 60)
-debugformat('literals', literals, 30)
-
+debugformats('literals', literals)
 
 lists = """
 - This is the first list item.
@@ -112,9 +120,7 @@ Line blocks are also a form of list:
 | This is the second line.
 """
 
-debugformat('lists', lists, 60)
-debugformat('lists', lists, 30)
-
+debugformats('lists', lists)
 
 options = """
 There is support for simple option lists,
@@ -140,9 +146,7 @@ marker after the option. It is treated as a normal paragraph:
 --foo bar baz
 """
 
-debugformat('options', options, 60)
-debugformat('options', options, 30)
-
+debugformats('options', options)
 
 fields = """
 :a: First item.
@@ -155,8 +159,7 @@ Next list:
 :much too large: This key is big enough to get its own line.
 """
 
-debugformat('fields', fields, 60)
-debugformat('fields', fields, 30)
+debugformats('fields', fields)
 
 containers = """
 Normal output.
@@ -174,14 +177,14 @@ Normal output.
       Debug output.
 """
 
-debugformat('containers (normal)', containers, 60)
-debugformat('containers (verbose)', containers, 60, keep=['verbose'])
-debugformat('containers (debug)', containers, 60, keep=['debug'])
-debugformat('containers (verbose debug)', containers, 60,
+debugformats('containers (normal)', containers)
+debugformats('containers (verbose)', containers, keep=['verbose'])
+debugformats('containers (debug)', containers, keep=['debug'])
+debugformats('containers (verbose debug)', containers,
             keep=['verbose', 'debug'])
 
 roles = """Please see :hg:`add`."""
-debugformat('roles', roles, 60)
+debugformats('roles', roles)
 
 
 sections = """
@@ -197,11 +200,12 @@ Subsection
 Markup: ``foo`` and :hg:`help`
 ------------------------------
 """
-debugformat('sections', sections, 20)
+debugformats('sections', sections)
 
 
 admonitions = """
 .. note::
+
    This is a note
 
    - Bullet 1
@@ -214,7 +218,7 @@ admonitions = """
    This is danger
 """
 
-debugformat('admonitions', admonitions, 30)
+debugformats('admonitions', admonitions)
 
 comments = """
 Some text.
@@ -230,4 +234,27 @@ Some text.
 Empty comment above
 """
 
-debugformat('comments', comments, 30)
+debugformats('comments', comments)
+
+
+data = [['a', 'b', 'c'],
+         ['1', '2', '3'],
+         ['foo', 'bar', 'baz this list is very very very long man']]
+
+rst = minirst.maketable(data, 2, True)
+table = ''.join(rst)
+
+print(table)
+
+debugformats('table', table)
+
+data = [['s', 'long', 'line\ngoes on here'],
+        ['', 'xy', 'tried to fix here\n        by indenting']]
+
+rst = minirst.maketable(data, 1, False)
+table = ''.join(rst)
+
+print(table)
+
+debugformats('table+nl', table)
+
